#!/bin/bash
#
# install.sh - install script for bootsplash progress bar support
#
#

SRC_DIR="${PWD}/src"
VRS_DIR="${SRC_DIR}/versions"

SRC_FILES="	/sbin/splash
			/sbin/bootanim
			/etc/init.d/bootsplash
			/etc/conf.d/bootsplash.conf"
VRS_FILES="	/sbin/rc
			/sbin/runscript.sh
			/sbin/functions.sh
			/etc/init.d/halt.sh"

if [ "$UID" -ne "0" ]; then
	echo "$0 must be started as root!"
	echo "Exiting..."
	exit 1
fi
#
# Check for epm
#
if [ ! -x /usr/bin/epm ]; then
	echo "$0 depends on package 'epm'"
	echo "Please run $0 a second time after you have emerged 'epm'"
	exit 1
fi
#
# Check baselayout version
#
BVERSION="`epm -q baselayout | cut -d '-' -f 2-`"
#
# Get patch versions and compare them to the baselayout version
#
for version in ${VRS_DIR}/*; do
	version=${version##*/}
	if [ "${BVERSION}" == "$version" ]; then
		echo "Your baselayout version is '$BVERSION', taking files out of"
		echo "directory '${VRS_DIR}/${version}'"
		echo
		found="true"
		break;
	fi
done 

if [ -z "$found" ]; then
	cat << EOF
*****************************************************************
* WARNING: You do not have any version of the baselayout package 
* supported by this script. The versions should match, because I
* have to patch some files out of the baselayout package
* (/sbin/rc, /sbin/functions.sh, /sbin/runscript.sh, /etc/init.d/halt.sh).
* If you want to proceed, I will make a backup of each file
* (/sbin/rc~, /sbin/functions.sh~, /sbin/runscript.sh~ /etc/init.d/halt.sh~).
******************************************************************

EOF
	echo -n "Should I proceed ? (y/n): "
	read input
	[ "$input" != "y" ] && exit 0

	#
	# Touch the numbered versions (simplifies the output) 
	#
	mkdir .versions 2>/dev/null
	num=0
	for version in ${VRS_DIR}/*; do
		version=${version##*/}
		num=$(($num + 1))
		touch ".versions/${num}) ${version}"
	done

	echo
	echo "Ok, your baselayout version is '$BVERSION'"
	echo "Please choose a patch version that is close"
	echo "to your baselayout version from the list below:"
	echo
	cd .versions && ls *\)* && cd ../
	echo
	declare -i input
	while true; do
		echo -n "Which patch should I use ? (1-$num): "
		read input

		if [ $input -lt 1 -o $input -gt $num ]; then
			echo "  $input out of range, please try again"
		else
			break
		fi
	done
	echo
	#
	# Get patch version
	#
	version="`ls .versions/${input}\)* | cut -d ' ' -f 2 `"
fi
#
# Move /sbin/splash to /sbin/splash.bin if it's not
# already the wrapper script
#
if [ -z "`cat /sbin/splash 2>/dev/null | grep \#\!\/bin\/bash`" ]; then
	echo -n "Moving /sbin/splash to /sbin/splash.bin: "
	result="`mv /sbin/splash /sbin/splash.bin 2>&1`"
	if [ $? -eq 0 ]; then
		echo "Ok"
	else
		echo "Failed"
		echo "!!! $result !!!"
	fi
fi
#
# Install version independend files
#
echo "Installing truetype font(s):"
	mkdir -p /etc/bootsplash
	cp -v ./ttfs/* /etc/bootsplash/

echo "Installing files:"
#
# we have different files for 1.8.5* and 1.8.6*
#
minor=$(echo $version | cut -d. -f 3)

for file in $SRC_FILES; do
	base="${file##*/}"
	test -f "${SRC_DIR}/${base}.${minor}" && \
	cp -v "${SRC_DIR}/${base}.${minor}" "${SRC_DIR}/${base}"
	install -v -D -b "${SRC_DIR}/${base}" "$file"
done
#
# Install baselayout specific files
#
for file in $VRS_FILES; do
	base="${file##*/}"
	install -v -D -b "${VRS_DIR}/${version}/${base}" "$file"
done

echo "Adding bootsplash to runlevel 'default': "
	rc-update add bootsplash default 
	
cat << EOF

Please read the README and INSTALL files for further
information (e.g. how to get animations to work)

Installation completed. Have fun !
EOF

exit 0
